@@ -1,5 +1,5 @@
 package CPAN::Mini;
-our $VERSION = '0.36';
+our $VERSION = '0.38';
 
 use strict;
 use warnings;
@@ -10,9 +10,9 @@ CPAN::Mini - create a minimal mirror of CPAN
 
 =head1 VERSION
 
-version 0.36
+version 0.38
 
- $Id: Mini.pm,v 1.18 2005/01/06 23:40:22 rjbs Exp $
+ $Id: /my/rjbs/code/minicpan/trunk/lib/CPAN/Mini.pm 15281 2005-10-13T04:34:49.264423Z rjbs  $
 
 =head1 SYNOPSIS
 
@@ -103,6 +103,10 @@ and ponie.
 
 If true, CPAN::Mini will print status messages to STDOUT as it works.
 
+=item * C<errors>
+
+If true, CPAN::Mini will warn with status messages on errors.  (default: true)
+
 =item * C<path_filters>
 
 This options provides a set of rules for filtering paths.  If a distribution
@@ -139,9 +143,6 @@ sub update_mirror {
 	my $self  = shift;
 	$self = $self->new(@_) unless ref $self;
 
-	croak "no local mirror supplied"  unless $self->{local};
-	croak "no remote mirror supplied" unless $self->{remote};
-
 	# mirrored tracks the already done, keyed by filename
 	# 1 = local-checked, 2 = remote-mirrored
 	$self->mirror_indices;
@@ -182,9 +183,23 @@ above, under C<update_mirror>.
 
 sub new {
 	my $class = shift;
-	my %defaults = (changes_made => 0, dirmode => 0711, mirrored => {});
+	my %defaults = (
+    changes_made => 0,
+    dirmode      => 0711,
+    errors       => 1,
+    mirrored     => {}
+  );
 
-	bless { %defaults, @_ } => $class;
+	my $self = bless { %defaults, @_ } => $class;
+
+	croak "no local mirror supplied"  unless $self->{local};
+  croak "no write permission to local mirror" unless -w $self->{local};
+
+	croak "no remote mirror supplied" unless $self->{remote};
+  croak "unable to contact the remote mirror"
+    unless LWP::Simple::head($self->{remote});
+
+	return $self;
 }
 
 =head2 C<< mirror_indices >>
@@ -215,8 +230,12 @@ sub mirror_file {
 	my $path   = shift;           # partial URL
 	my $skip_if_present = shift;  # true/false
 
-	my $remote_uri = URI->new_abs($path, $self->{remote})->as_string; # full URL
-	my $local_file = catfile($self->{local}, split "/", $path); # native absolute file
+  # full URL
+	my $remote_uri = URI->new_abs($path, $self->{remote})->as_string;
+
+  # native absolute file
+	my $local_file = catfile($self->{local}, split "/", $path);
+
 	my $checksum_might_be_up_to_date = 1;
 
 	if ($skip_if_present and -f $local_file) {
@@ -235,7 +254,8 @@ sub mirror_file {
 			$self->trace(" ... updated\n");
 			$self->{changes_made}++;
 		} elsif ($status != RC_NOT_MODIFIED) {
-			warn "\n$remote_uri: $status\n";
+			warn( ($self->{trace} ? "\n" : '')
+        . "$remote_uri: $status\n") if $self->{errors};
 			return;
 		} else {
 			$self->trace(" ... up to date\n");
@@ -314,8 +334,9 @@ sub file_allowed {
 
 =head2 C<< clean_unmirrored >>
 
-This method finds any files in the local mirror which are no longer needed and
-calls the C<clean_file> method on them.
+This method looks through the local mirror's files.  If it finds a file that
+neither belongs in the mirror nor is allowed (see the C<file_allowed> method),
+C<clean_file> is called on the file.
 
 =cut
 
@@ -324,25 +345,32 @@ sub clean_unmirrored {
 
 	find sub {
 		my $file = canonpath($File::Find::name);
-		$self->clean_file($file);
+    return unless (-f $file and not $self->{mirrored}{$file});
+    return if $self->file_allowed($file);
+    $self->trace("cleaning $file ...");
+		if ($self->clean_file($file)) {
+      $self->trace("done\n");
+    } else {
+      $self->trace("couldn't be cleaned\n");
+    }
 	}, $self->{local};
 }
 
 =head2 C<< clean_file($filename) >>
 
-This method, called by C<clean_unmirrored>, checks whether the named file
-exists.  If it exists, and was not mirrored, and C<file_allowed> doesn't say it
-can stay, the file is deleted.
+This method, called by C<clean_unmirrored>, deletes the named file.  It returns
+true if the file is successfully unlinked.  Otherwise, it returns false.
 
 =cut
 
 sub clean_file {
 	my ($self, $file) = @_;
 
-	return unless (-f $file and not $self->{mirrored}{$file});
-	return if $self->file_allowed($file);
-	unless (unlink $file) { warn "$file ... cannot be removed: $!"; return; }
-	$self->trace("$file ... removed\n");
+	unless (unlink $file) {
+    warn "$file ... cannot be removed: $!" if $self->{errors};
+    return;
+  }
+  return 1;
 }
 
 =head2 C<< trace( $message, $force ) >>
